@@ -23,8 +23,10 @@ Usage: $0 [OPTIONS]
         --perconaft_branch  Branch for PerconaFT
         --tokubackup_repo   TokuBackup repo
         --tokubackup_branch Btanch for TokuBackup
+        --rpm_release       RPM version( default = 1)
+        --deb_release       DEB version( default = 1)
         --help) usage ;;
-Example $0 --builddir=/tmp/PS56 --get_sources=1 --build_src_rpm=1 --build_rpm=1
+Example $0 --builddir=/tmp/PS57 --get_sources=1 --build_src_rpm=1 --build_rpm=1
 EOF
         exit 1
 }
@@ -60,6 +62,8 @@ parse_arguments() {
             --tokubackup_branch=*) 	TOKUBACKUP_BRANCH="$val" ;;
             --perconaft_repo=*) PERCONAFT_REPO="$val" ;;
             --tokubackup_repo=*) TOKUBACKUP_REPO="$val" ;;
+            --rpm_release=*) RPM_RELEASE="$val" ;;
+            --deb_release=*) DEB_RELEASE="$val" ;;
             --help) usage ;;      
             *)
               if test -n "$pick_args"
@@ -89,34 +93,20 @@ check_workdir(){
 add_percona_yum_repo(){
     if [ ! -f /etc/yum.repos.d/percona-dev.repo ]
     then
-        cat >/etc/yum.repos.d/percona-dev.repo <<EOL
-[percona-dev-$basearch]
-name=Percona internal YUM repository for build slaves \$releasever - \$basearch
-baseurl=http://jenkins.percona.com/yum-repo/\$releasever/RPMS/\$basearch
-gpgkey=http://jenkins.percona.com/yum-repo/PERCONA-PACKAGING-KEY
-gpgcheck=0
-enabled=1
-
-[percona-dev-noarch]
-name=Percona internal YUM repository for build slaves \$releasever - noarch
-baseurl=http://jenkins.percona.com/yum-repo/\$releasever/RPMS/noarch
-gpgkey=http://jenkins.percona.com/yum-repo/PERCONA-PACKAGING-KEY
-gpgcheck=0
-enabled=1
-EOL
+        curl -o /etc/yum.repos.d/ https://jenkins.percona.com/yum-repo/percona-dev.repo
     fi
     return
 }
 
 add_percona_apt_repo(){
   if [ ! -f /etc/apt/sources.list.d/percona-dev.list ]; then
-    cat >/etc/apt/sources.list.d/percona-dev.list <<EOL
-deb http://jenkins.percona.com/apt-repo/ @@DIST@@ main
-deb-src http://jenkins.percona.com/apt-repo/ @@DIST@@ main
-EOL
+    curl -o /etc/apt/sources.list.d/ https://jenkins.percona.com/apt-repo/percona-dev.list.template
+    mv /etc/apt/sources.list.d/percona-dev.list.template /etc/apt/sources.list.d/percona-dev.list
     sed -i "s:@@DIST@@:$OS_NAME:g" /etc/apt/sources.list.d/percona-dev.list
   fi
-  wget -qO - http://jenkins.percona.com/apt-repo/8507EFA5.pub | apt-key add -
+ 
+  wget -q -O - http://jenkins.percona.com/apt-repo/8507EFA5.pub | sudo apt-key add -
+  wget -q -O - http://jenkins.percona.com/apt-repo/CD2EFD2A.pub | sudo apt-key add -
   return
 }
 
@@ -148,36 +138,46 @@ get_sources(){
     git reset --hard
     #
     source VERSION
-    cat VERSION > ../percona-server-5.6.properties
-    echo "REVISION=${REVISION}" >> ../percona-server-5.6.properties
+    cat VERSION > ../percona-server-5.7.properties
+    echo "REVISION=${REVISION}" >> ../percona-server-5.7.properties
     BRANCH_NAME="${BRANCH}"
-    echo "BRANCH_NAME=${BRANCH_NAME}" >> ../percona-server-5.6.properties
+    echo "BRANCH_NAME=${BRANCH_NAME}" >> ../percona-server-5.7.properties
     export PRODUCT=Percona-Server-${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}
-    echo "PRODUCT=Percona-Server-${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}" >> ../percona-server-5.6.properties
+    echo "PRODUCT=Percona-Server-${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}" >> ../percona-server-5.7.properties
     export PRODUCT_FULL=${PRODUCT}.${MYSQL_VERSION_PATCH}${MYSQL_VERSION_EXTRA}
-    echo "PRODUCT_FULL=${PRODUCT}.${MYSQL_VERSION_PATCH}${MYSQL_VERSION_EXTRA}" >> ../percona-server-5.6.properties
-    echo "BUILD_NUMBER=${BUILD_NUMBER}" >> ../percona-server-5.6.properties
-    echo "BUILD_ID=${BUILD_ID}" >> ../percona-server-5.6.properties
-    echo "PERCONAFT_REPO=${PERCONAFT_REPO}" >> ../percona-server-5.6.properties
-    echo "PERCONAFT_BRANCH=${PERCONAFT_BRANCH}" >> ../percona-server-5.6.properties
-    echo "TOKUBACKUP_REPO=${TOKUBACKUP_REPO}" >> ../percona-server-5.6.properties
-    echo "TOKUBACKUP_BRANCH=${TOKUBACKUP_BRANCH}" >> ../percona-server-5.6.properties
-    echo "TOKUDB_VERSION=${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}.${MYSQL_VERSION_PATCH}${MYSQL_VERSION_EXTRA}" >> ../percona-server-5.6.properties
-    #
-    if [ -z "${DESTINATION}" ]; then
+    echo "PRODUCT_FULL=${PRODUCT}.${MYSQL_VERSION_PATCH}${MYSQL_VERSION_EXTRA}" >> ../percona-server-5.7.properties
+    echo "BUILD_NUMBER=${BUILD_NUMBER}" >> ../percona-server-5.7.properties
+    echo "BUILD_ID=${BUILD_ID}" >> ../percona-server-5.7.properties
+    echo "PERCONAFT_REPO=${PERCONAFT_REPO}" >> ../percona-server-5.7.properties
+    echo "PERCONAFT_BRANCH=${PERCONAFT_BRANCH}" >> ../percona-server-5.7.properties
+    echo "TOKUBACKUP_REPO=${TOKUBACKUP_REPO}" >> ../percona-server-5.7.properties
+    echo "TOKUBACKUP_BRANCH=${TOKUBACKUP_BRANCH}" >> ../percona-server-5.7.properties
+    echo "TOKUDB_VERSION=${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}.${MYSQL_VERSION_PATCH}${MYSQL_VERSION_EXTRA}" >> ../percona-server-5.7.properties
+    BOOST_PACKAGE_NAME=$(cat cmake/boost.cmake|grep "SET(BOOST_PACKAGE_NAME"|awk -F '"' '{print $2}')
+    echo "BOOST_PACKAGE_NAME=${BOOST_PACKAGE_NAME}" >> ../percona-server-5.7.properties
+    echo "RPM_RELEASE=${RPM_RELEASE}" >> ../percona-server-5.7.properties
+    echo "DEB_RELEASE=${DEB_RELEASE}" >> ../percona-server-5.7.properties
+
+    if [ -z "${DESTINATION:-}" ]; then
         export DESTINATION=experimental
     fi
-    #
     TIMESTAMP=$(date "+%Y%m%d-%H%M%S")
-    echo "DESTINATION=${DESTINATION}" >> ../percona-server-5.6.properties
-    echo "UPLOAD=UPLOAD/${DESTINATION}/BUILDS/${PRODUCT}/${PRODUCT_FULL}/${BRANCH_NAME}/${REVISION}/${TIMESTAMP}" >> ../percona-server-5.6.properties
-    #
-    # initialize git submodules
+    echo "DESTINATION=${DESTINATION}" >> ../percona-server-5.7.properties
+    echo "UPLOAD=UPLOAD/${DESTINATION}/BUILDS/${PRODUCT}/${PRODUCT_FULL}/${BRANCH_NAME}/${REVISION}/${TIMESTAMP}" >> ../percona-server-5.7.properties
+
     rm -rf storage/tokudb/PerconaFT
     rm -rf plugin/tokudb-backup-plugin/Percona-TokuBackup
-    git submodule deinit -f .
+    git submodule init
+    git submodule update
     rm -rf storage/tokudb/PerconaFT
     rm -rf plugin/tokudb-backup-plugin/Percona-TokuBackup
+    if [ ${PERCONAFT_REPO} = 0 ]; then
+        PERCONAFT_REPO=''
+    fi
+    if [ ${TOKUBACKUP_REPO} = 0 ]; then
+        TOKUBACKUP_REPO=''
+    fi
+
     if [ -z ${PERCONAFT_REPO} -a -z ${TOKUBACKUP_REPO} ]; then
         mkdir plugin/tokudb-backup-plugin/Percona-TokuBackup
         mkdir storage/tokudb/PerconaFT
@@ -189,33 +189,34 @@ get_sources(){
         if [ ${PERCONAFT_BRANCH} = "master" ]; then
             git pull
         fi
-        cd $WORKDIR/percona-server
+        cd ${WORKDIR}/percona-server
+        #
         cd plugin/tokudb-backup-plugin/Percona-TokuBackup
         git fetch origin
         git checkout ${TOKUBACKUP_BRANCH}
         if [ ${TOKUBACKUP_BRANCH} = "master" ]; then
             git pull
         fi
-        cd $WORKDIR/percona-server
+        cd ${WORKDIR}/percona-server
     else
         cd storage/tokudb
         git clone ${PERCONAFT_REPO}
         cd PerconaFT
         git checkout ${PERCONAFT_BRANCH}
-        cd ../
         cd ${WORKDIR}/percona-server
+        #
         cd plugin/tokudb-backup-plugin
         git clone ${TOKUBACKUP_REPO}
         cd Percona-TokuBackup
         git checkout ${TOKUBACKUP_BRANCH}
-        cd ../
         cd ${WORKDIR}/percona-server
     fi
     #
+    git submodule update
     cmake . -DDOWNLOAD_BOOST=1 -DWITH_BOOST=${WORKDIR}/build-ps/boost
     make dist
     #
-    EXPORTED_TAR=$(basename $(find . -type f -name *.tar.gz | sort | tail -n 1))
+    EXPORTED_TAR=$(basename $(find . -type f -name percona-server*.tar.gz | sort | tail -n 1))
     #
     PSDIR=${EXPORTED_TAR%.tar.gz}
     rm -fr ${PSDIR}
@@ -224,6 +225,9 @@ get_sources(){
     # add git submodules because make dist uses git archive which doesn't include them
     rsync -av storage/tokudb/PerconaFT ${PSDIR}/storage/tokudb --exclude .git
     rsync -av plugin/tokudb-backup-plugin/Percona-TokuBackup ${PSDIR}/plugin/tokudb-backup-plugin --exclude .git
+    rsync -av storage/rocksdb/rocksdb/ ${PSDIR}/storage/rocksdb/rocksdb --exclude .git
+    rsync -av storage/rocksdb/third_party/lz4/ ${PSDIR}/storage/rocksdb/third_party/lz4 --exclude .git
+    rsync -av storage/rocksdb/third_party/zstd/ ${PSDIR}/storage/rocksdb/third_party/zstd --exclude .git
     #
     cd ${PSDIR}
     # set tokudb version - can be seen with show variables like '%version%'
@@ -232,24 +236,22 @@ get_sources(){
     sed -i "s:@@PERCONA_VERSION_EXTRA@@:${MYSQL_VERSION_EXTRA#-}:g" build-ps/debian/rules
     sed -i "s:@@REVISION@@:${REVISION}:g" build-ps/debian/rules
     sed -i "s:@@TOKUDB_BACKUP_VERSION@@:${TOKUDB_VERSION}:g" build-ps/debian/rules
-    sed -i "s:@@PERCONA_VERSION_EXTRA@@:${MYSQL_VERSION_EXTRA#-}:g" build-ps/debian/rules.yakkety
-    sed -i "s:@@REVISION@@:${REVISION}:g" build-ps/debian/rules.yakkety
-    sed -i "s:@@TOKUDB_BACKUP_VERSION@@:${TOKUDB_VERSION}:g" build-ps/debian/rules.yakkety
-
     sed -i "s:@@PERCONA_VERSION_EXTRA@@:${MYSQL_VERSION_EXTRA#-}:g" build-ps/debian/rules.notokudb
     sed -i "s:@@REVISION@@:${REVISION}:g" build-ps/debian/rules.notokudb
     #
+    sed -i "s:@@PERCONA_VERSION_EXTRA@@:${MYSQL_VERSION_EXTRA#-}:g" build-ps/ubuntu/rules
+    sed -i "s:@@REVISION@@:${REVISION}:g" build-ps/ubuntu/rules
+    sed -i "s:@@TOKUDB_BACKUP_VERSION@@:${TOKUDB_VERSION}:g" build-ps/ubuntu/rules
+    sed -i "s:@@PERCONA_VERSION_EXTRA@@:${MYSQL_VERSION_EXTRA#-}:g" build-ps/ubuntu/rules.notokudb
+    sed -i "s:@@REVISION@@:${REVISION}:g" build-ps/ubuntu/rules.notokudb
+    #
     sed -i "s:@@MYSQL_VERSION@@:${MYSQL_VERSION_MAJOR}.${MYSQL_VERSION_MINOR}.${MYSQL_VERSION_PATCH}:g" build-ps/percona-server.spec
     sed -i "s:@@PERCONA_VERSION@@:${MYSQL_VERSION_EXTRA#-}:g" build-ps/percona-server.spec
     sed -i "s:@@REVISION@@:${REVISION}:g" build-ps/percona-server.spec
-    sed -i "s:@@TOKUDB_BACKUP_VERSION@@:${TOKUDB_VERSION}:g" build-ps/percona-server.spec
-    #
-    sed -i "s:@@TOKUDB_BACKUP_VERSION@@:${TOKUDB_VERSION}:g" build-ps/build-binary.sh
-    # create a PS tar
+    sed -i "s:@@RPM_RELEASE@@:${RPM_RELEASE}:g" build-ps/percona-server.spec
+    sed -i "s:@@BOOST_PACKAGE_NAME@@:${BOOST_PACKAGE_NAME}:g" build-ps/percona-server.spec
     cd ${WORKDIR}/percona-server
     tar --owner=0 --group=0 --exclude=.bzr --exclude=.git -czf ${PSDIR}.tar.gz ${PSDIR}
-    rm -fr ${PSDIR}
-
     
     mkdir $WORKDIR/source_tarball
     mkdir $CURDIR/source_tarball
@@ -280,7 +282,7 @@ install_deps() {
         echo "Dependencies will not be installed"
         return;
     fi
-    if [ ! $( id -u ) -eq 0 ]
+    if [ $( id -u ) -ne 0 ]
     then
         echo "It is not possible to instal dependencies. Please run as root"
         exit 1
@@ -291,42 +293,43 @@ install_deps() {
         RHEL=$(rpm --eval %rhel)
         ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
         add_percona_yum_repo
-	yum -y install https://repo.percona.com/yum/percona-release-1.0-11.noarch.rpm || true
-        yum -y install epel-release git redhat-lsb
-        yum clean metadata
-        yum -y install libtool rpm-build gcc-c++ gperf ncurses-devel perl readline-devel openssl-devel jemalloc 
+        yum -y install https://repo.percona.com/yum/percona-release-latest.noarch.rpm || true
+        percona-release enable origin release
+        yum -y install epel-release
+        yum -y install git numactl-devel rpm-build gcc-c++ gperf ncurses-devel perl readline-devel openssl-devel jemalloc 
         yum -y install time zlib-devel libaio-devel bison cmake pam-devel libeatmydata jemalloc-devel
-        yum -y install perl-Time-HiRes numactl-devel || true
-        yum -y install selinux-policy-devel || true
-	    if [ ${RHEL} -lt 7 ]; then
-	        if [ $(uname -m) = x86_64 ]; then
-               yum install -y percona-devtoolset-gcc percona-devtoolset-binutils percona-devtoolset-gcc-c++ percona-devtoolset-libstdc++-devel percona-devtoolset-valgrind-devel
-	        else
-               wget -O /etc/yum.repos.d/slc6-devtoolset.repo http://linuxsoft.cern.ch/cern/devtoolset/slc6-devtoolset.repo
-               wget -O /etc/pki/rpm-gpg/RPM-GPG-KEY-cern https://raw.githubusercontent.com/cms-sw/cms-docker/master/slc6-vanilla/RPM-GPG-KEY-cern
-               yum -y install  devtoolset-2-gcc-c++ devtoolset-2-binutils libevent2-devel
-            fi
-            yum -y install Percona-Server-shared-56
+        yum -y install perl-Time-HiRes libcurl-devel openldap-devel unzip wget libcurl-devel 
+        yum -y install perl-Env perl-Data-Dumper perl-JSON MySQL-python perl-Digest perl-Digest-MD5 perl-Digest-Perl-MD5 || true
+        if [ ${RHEL} -lt 7 -a $(uname -m) = x86_64 ]; then
+            yum -y install percona-devtoolset-gcc percona-devtoolset-gcc-c++ percona-devtoolset-binutils 
+        fi
+        if [ "x$RHEL" = "x6" ]; then
+            yum -y install Percona-Server-shared-56  
         fi
     else
         apt-get -y install dirmngr || true
         add_percona_apt_repo
       	apt-get update
         apt-get -y install dirmngr || true
-        apt-get -y install lsb-release
+        apt-get -y install lsb-release wget
+        export DEBIAN_FRONTEND="noninteractive"
+        export DIST="$(lsb_release -sc)"
+	    until sudo apt-get update; do
+    	    sleep 1
+            echo "waiting"
+        done
         apt-get -y purge eatmydata || true
-    	apt-get -y install git libtool curl bison cmake perl libssl-dev gcc g++ libaio-dev libldap2-dev libwrap0-dev gdb
-  	apt-get -y install lsb-release libmecab-dev libncurses5-dev libreadline-dev libpam-dev zlib1g-dev 
-  	apt-get -y install libnuma-dev libjemalloc-dev libeatmydata
+        apt-get update
+        apt-get -y install psmisc
+        apt-get -y install libsasl2-modules:amd64 || apt-get -y install libsasl2-modules
+        apt-get -y install dh-systemd || true
+        apt-get -y install curl bison cmake perl libssl-dev gcc g++ libaio-dev libldap2-dev libwrap0-dev gdb unzip gawk
+        apt-get -y install lsb-release libmecab-dev libncurses5-dev libreadline-dev libpam-dev zlib1g-dev libcurl4-openssl-dev
+        apt-get -y install libldap2-dev libnuma-dev libjemalloc-dev libeatmydata libc6-dbg valgrind libjson-perl python-mysqldb libsasl2-dev
+
+        apt-get -y install libmecab2 mecab mecab-ipadic
+        apt-get -y install build-essential devscripts
         apt-get -y install cmake autotools-dev autoconf automake build-essential devscripts debconf debhelper fakeroot 
-        apt-get -y install chrpath ghostscript gawk bison cmake libaio-dev libmecab-dev libncurses5-dev lsb-release perl po-debconf psmisc zlib1g-dev libreadline-dev libpam-dev libssl-dev libnuma-dev libwrap0-dev libldap2-dev dh-systemd || true
-    fi
-    if [ ! -d /usr/local/percona-subunit2junitxml ]; then
-        cd /usr/local
-        git clone https://github.com/percona/percona-subunit2junitxml.git
-        rm -rf /usr/bin/subunit2junitxml
-        ln -s /usr/local/percona-subunit2junitxml/subunit2junitxml /usr/bin/subunit2junitxml
-        cd ${CURPLACE}
     fi
     return;
 }
@@ -395,17 +398,25 @@ build_srpm(){
     TMPREL=$(echo ${TARFILE}| awk -F '-' '{print $4}')
     RELEASE=${TMPREL%.tar.gz}
     #
+    mkdir -vp rpmbuild/{SOURCES,SPECS,BUILD,SRPMS,RPMS}
+    #
     cd ${WORKDIR}/rpmbuild/SPECS
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/*.spec' --strip=2
     #
     cd ${WORKDIR}/rpmbuild/SOURCES
+    #wget http://downloads.sourceforge.net/boost/${BOOST_PACKAGE_NAME}.tar.bz2
+    wget http://jenkins.percona.com/downloads/boost/${BOOST_PACKAGE_NAME}.tar.gz
     tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/*.patch' --strip=3
+    tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/filter-provides.sh' --strip=3
+    tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/filter-requires.sh' --strip=3
+    tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/mysql_config.sh' --strip=3
+    tar vxzf ${WORKDIR}/${TARFILE} --wildcards '*/build-ps/rpm/my_config.h' --strip=3
     #
     cd ${WORKDIR}
     #
     mv -fv ${TARFILE} ${WORKDIR}/rpmbuild/SOURCES
-
-    rpmbuild -bs --define "_topdir ${WORKDIR}/rpmbuild" --with tokudb --define "dist .generic" --define "pkg_ver .${RPM_RELEASE}" rpmbuild/SPECS/percona-server.spec
+    #
+    rpmbuild -bs --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .generic" rpmbuild/SPECS/percona-server.spec
     #
 
     mkdir -p ${WORKDIR}/srpm
@@ -415,6 +426,49 @@ build_srpm(){
     return
 }
 
+build_mecab_lib(){
+    MECAB_TARBAL="mecab-0.996.tar.gz"
+    MECAB_LINK="http://jenkins.percona.com/downloads/mecab/${MECAB_TARBAL}"
+    MECAB_DIR="${WORKDIR}/${MECAB_TARBAL%.tar.gz}"
+    MECAB_INSTALL_DIR="${WORKDIR}/mecab-install"
+    rm -f ${MECAB_TARBAL}
+    rm -rf ${MECAB_DIR}
+    rm -rf ${MECAB_INSTALL_DIR}
+    mkdir ${MECAB_INSTALL_DIR}
+    wget ${MECAB_LINK}
+    tar xf ${MECAB_TARBAL}
+    cd ${MECAB_DIR}
+    ./configure --with-pic --prefix=/usr
+    make
+    make check
+    make DESTDIR=${MECAB_INSTALL_DIR} install
+    cd ${WORKDIR}
+}
+
+build_mecab_dict(){
+    MECAB_IPADIC_TARBAL="mecab-ipadic-2.7.0-20070801.tar.gz"
+    MECAB_IPADIC_LINK="http://jenkins.percona.com/downloads/mecab/${MECAB_IPADIC_TARBAL}"
+    MECAB_IPADIC_DIR="${WORKDIR}/${MECAB_IPADIC_TARBAL%.tar.gz}"
+    rm -f ${MECAB_IPADIC_TARBAL}
+    rm -rf ${MECAB_IPADIC_DIR}
+    wget ${MECAB_IPADIC_LINK}
+    tar xf ${MECAB_IPADIC_TARBAL}
+    cd ${MECAB_IPADIC_DIR}
+    # these two lines should be removed if proper packages are created and used for builds
+    export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:${MECAB_INSTALL_DIR}/usr/lib
+    sed -i "/MECAB_DICT_INDEX=\"/c\MECAB_DICT_INDEX=\"${MECAB_INSTALL_DIR}\/usr\/libexec\/mecab\/mecab-dict-index\"" configure
+    #
+    ./configure --with-mecab-config=${MECAB_INSTALL_DIR}/usr/bin/mecab-config
+    make
+    make DESTDIR=${MECAB_INSTALL_DIR} install
+    cd ../
+    cd ${MECAB_INSTALL_DIR}
+    if [ -d usr/lib64 ]; then
+        mv usr/lib64/* usr/lib
+    fi
+    cd ${WORKDIR}
+}
+
 build_rpm(){
     if [ $RPM = 0 ]
     then
@@ -449,21 +503,30 @@ build_rpm(){
     RHEL=$(rpm --eval %rhel)
     ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
     #
-    echo "RHEL=${RHEL}" >> percona-server-5.6.properties
-    echo "ARCH=${ARCH}" >> percona-server-5.6.properties
+    echo "RHEL=${RHEL}" >> percona-server-5.7.properties
+    echo "ARCH=${ARCH}" >> percona-server-5.7.properties
     #
-    if [ ${RHEL} = 6 ]; then
-        if [ ${ARCH} = x86_64 ]; then
+    SRCRPM=$(basename $(find . -name '*.src.rpm' | sort | tail -n1))
+    mkdir -vp rpmbuild/{SOURCES,SPECS,BUILD,SRPMS,RPMS}
+    #
+    mv *.src.rpm rpmbuild/SRPMS
+    #
+    if [ ${ARCH} = x86_64 ]; then
+        if [ ${RHEL} != 7 ]; then
             source /opt/percona-devtoolset/enable
-            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --with tokudb --define "dist .el${RHEL}" --define "pkg_ver .${RPM_RELEASE}" --rebuild rpmbuild/SRPMS/${SRC_RPM}
-	else
-            source /opt/rh/devtoolset-2/enable
-            rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "pkg_ver .${RPM_RELEASE}" --rebuild rpmbuild/SRPMS/${SRC_RPM}
         fi
-    else
-        rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --with tokudb --define "dist .el${RHEL}" --define "pkg_ver .${RPM_RELEASE}" --rebuild rpmbuild/SRPMS/${SRC_RPM}
     fi
 
+    build_mecab_lib
+    build_mecab_dict
+
+    cd ${WORKDIR}
+    #
+    if [ ${ARCH} = x86_64 ]; then
+        rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+    else
+        rpmbuild --define "_topdir ${WORKDIR}/rpmbuild" --define "dist .el${RHEL}" --define "with_tokudb 0" --define "with_rocksdb 0" --define "with_mecab ${MECAB_INSTALL_DIR}/usr" --rebuild rpmbuild/SRPMS/${SRCRPM}
+    fi
     return_code=$?
     if [ $return_code != 0 ]; then
         exit $return_code
@@ -492,25 +555,23 @@ build_source_deb(){
     #
 
     TARFILE=$(basename $(find . -name 'percona-server-*.tar.gz' | grep -v tokudb | sort | tail -n1))
+
     NAME=$(echo ${TARFILE}| awk -F '-' '{print $1"-"$2}')
     VERSION=$(echo ${TARFILE}| awk -F '-' '{print $3}')
     SHORTVER=$(echo ${VERSION} | awk -F '.' '{print $1"."$2}')
     TMPREL=$(echo ${TARFILE}| awk -F '-' '{print $4}')
     RELEASE=${TMPREL%.tar.gz}
-    #
-  #  rm -fr ${NAME}-${VERSION}-${RELEASE}
-    #
+
     NEWTAR=${NAME}-${SHORTVER}_${VERSION}-${RELEASE}.orig.tar.gz
     mv ${TARFILE} ${NEWTAR}
-    #
+
     tar xzf ${NEWTAR}
     cd ${NAME}-${VERSION}-${RELEASE}
     cp -ap build-ps/debian/ .
-    dch -D unstable --force-distribution -v "${VERSION}-${RELEASE}-1" "Update to new upstream release Percona Server ${VERSION}-${RELEASE}-2"
+    dch -D unstable --force-distribution -v "${VERSION}-${RELEASE}-${DEB_RELEASE}" "Update to new upstream release Percona Server ${VERSION}-${RELEASE}-1"
     dpkg-buildpackage -S
-    #
+
     cd ${WORKDIR}
-  #  rm -fr ${NAME}-${VERSION}-${RELEASE}
 
     mkdir -p $WORKDIR/source_deb
     mkdir -p $CURDIR/source_deb
@@ -541,39 +602,38 @@ build_deb(){
     done
     cd $WORKDIR
     rm -fv *.deb
-    #
-    export DEBIAN_VERSION=$(lsb_release -sc)
-    export ARCH=$(echo $(uname -m) | sed -e 's:i686:i386:g')
-    #
-    rm -fv *.deb
 
     export DEBIAN_VERSION="$(lsb_release -sc)"
+
     DSC=$(basename $(find . -name '*.dsc' | sort | tail -n 1))
-    DIRNAME=$(echo ${DSC%-1.dsc} | sed -e 's:_:-:g')
+    DIRNAME=$(echo ${DSC%-${DEB_RELEASE}.dsc} | sed -e 's:_:-:g')
     VERSION=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $4}')
     RELEASE=$(echo ${DSC} | sed -e 's:_:-:g' | awk -F'-' '{print $5}')
     ARCH=$(uname -m)
     export EXTRAVER=${MYSQL_VERSION_EXTRA#-}
     #
-    echo "ARCH=${ARCH}" >> percona-server-5.6.properties
-    echo "DEBIAN_VERSION=${DEBIAN_VERSION}" >> percona-server-5.6.properties
-    echo VERSION=${VERSION} >> percona-server-5.6.properties
+    echo "ARCH=${ARCH}" >> percona-server-5.7.properties
+    echo "DEBIAN_VERSION=${DEBIAN_VERSION}" >> percona-server-5.7.properties
+    echo "VERSION=${VERSION}" >> percona-server-5.7.properties
     #
 
     dpkg-source -x ${DSC}
-    #mv percona-server-source-5.6.tar.gz ${DIRNAME}/debian
+
     cd ${DIRNAME}
     #
-    dch -b -m -D "$DEBIAN_VERSION" --force-distribution -v "${VERSION}-${RELEASE}-1.${DEBIAN_VERSION}" 'Update distribution'
-    #
-    #
+    if [ ${DEBIAN_VERSION} = xenial -o ${DEBIAN_VERSION} = artful -o ${DEBIAN_VERSION} = bionic -o ${DEBIAN_VERSION} = trusty ]; then
+        rm -rf debian
+        cp -r build-ps/ubuntu debian
+    fi
+    dch -b -m -D "$DEBIAN_VERSION" --force-distribution -v "${VERSION}-${RELEASE}-${DEB_RELEASE}.${DEBIAN_VERSION}" 'Update distribution'
+
     if [ ${ARCH} != "x86_64" ]; then
         rm -f debian/rules
         rm -f debian/control
         mv debian/rules.notokudb debian/rules
         mv debian/control.notokudb debian/control
     else
-        if [ ${DEBIAN_VERSION} != trusty -a ${DEBIAN_VERSION} != xenial -a ${DEBIAN_VERSION} != stretch -a ${DEBIAN_VERSION} != jessie -a ${DEBIAN_VERSION} != vivid -a ${DEBIAN_VERSION} != yakkety -a ${DEBIAN_VERSION} != artful -a ${DEBIAN_VERSION} != bionic -a ${DEBIAN_VERSION} != cosmic -a ${DEBIAN_VERSION} != disco -a ${DEBIAN_VERSION} != buster ]; then
+        if [ ${DEBIAN_VERSION} != trusty -a ${DEBIAN_VERSION} != xenial -a ${DEBIAN_VERSION} != jessie -a ${DEBIAN_VERSION} != stretch -a ${DEBIAN_VERSION} != artful -a ${DEBIAN_VERSION} != bionic ]; then
             gcc47=$(which gcc-4.7 2>/dev/null || true)
             if [ -x "${gcc47}" ]; then
                 export CC=gcc-4.7
@@ -586,16 +646,24 @@ build_deb(){
             fi
         fi
     fi
-    #
 
-    if [ ${DEBIAN_VERSION} = "artful" -o ${DEBIAN_VERSION} = "bionic" -o ${DEBIAN_VERSION} = "cosmic" -o ${DEBIAN_VERSION} = "disco" -o ${DEBIAN_VERSION} = "buster" ]; then
-        sed -i 's/export CFLAGS=/export CFLAGS=-Wimplicit-fallthrough=0 /' debian/rules
-        sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wimplicit-fallthrough=0 /' debian/rules
+    if [ ${DEBIAN_VERSION} = "xenial" ]; then
+        sed -i 's/export CFLAGS=/export CFLAGS=-Wno-error=date-time /' debian/rules 
+        sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wno-error=date-time /' debian/rules
     fi
 
+    if [ ${DEBIAN_VERSION} = "stretch" ]; then
+        sed -i 's/export CFLAGS=/export CFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time /' debian/rules 
+        sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time /' debian/rules
+    fi
+
+    if [ ${DEBIAN_VERSION} = "artful" -o ${DEBIAN_VERSION} = "bionic" ]; then
+        sed -i 's/export CFLAGS=/export CFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time /' debian/rules 
+        sed -i 's/export CXXFLAGS=/export CXXFLAGS=-Wno-error=deprecated-declarations -Wno-error=unused-function -Wno-error=unused-variable -Wno-error=unused-parameter -Wno-error=date-time /' debian/rules
+    fi
 
     dpkg-buildpackage -rfakeroot -uc -us -b
-    #
+
     cd ${WORKDIR}
     mkdir -p $CURDIR/deb
     mkdir -p $WORKDIR/deb
@@ -631,38 +699,21 @@ build_tarball(){
     TMPREL=$(echo ${TARFILE}| awk -F '-' '{print $4}')
     RELEASE=${TMPREL%.tar.gz}
     #
-    if [ -f /etc/redhat-release ]; then
-      if [ ${RHEL} = 6 ]; then
-        if [ ${ARCH} = x86_64 ]; then
-          source /opt/percona-devtoolset/enable
-        else
-          source /opt/rh/devtoolset-2/enable
-        fi
-      fi
-    fi
     export CFLAGS=$(rpm --eval %{optflags} | sed -e "s|march=i386|march=i686|g")
     export CXXFLAGS="${CFLAGS}"
     if [ "${YASSL}" = 0 ]; then
         if [ -f /etc/redhat-release ]; then
-            SSL_VER_TMP=$(yum list installed|grep -i openssl|head -n1|awk '{print $2}'|awk -F "-" '{print $1}'|sed 's/\.//g'|sed 's/[a-z]$//' | awk -F':' '{print $2}')
-            if [ -z "${SSL_VER_TMP}" ]; then
-                SSL_VER_TMP=$(yum list installed|grep -i openssl|head -n1|awk '{print $2}'|awk -F "-" '{print $1}'|sed 's/\.//g'|sed 's/[a-z]$//')
-            fi
-            if [[ ${SSL_VER_TMP} == 102 ]]; then
-                export SSL_VER=".ssl${SSL_VER_TMP}.rpm"
-            else
-                export SSL_VER=".ssl${SSL_VER_TMP}"
-            fi
+            SSL_VER_TMP=$(yum list installed|grep -i openssl|head -n1|awk '{print $2}'|awk -F "-" '{print $1}'|sed 's/\.//g'|sed 's/[a-z]$//')
+            export SSL_VER=".ssl${SSL_VER_TMP}"
         else
             SSL_VER_TMP=$(dpkg -l|grep -i libssl|grep -v "libssl\-"|head -n1|awk '{print $2}'|awk -F ":" '{print $1}'|sed 's/libssl/ssl/g'|sed 's/\.//g')
-            if [[ ${SSL_VER_TMP} == 'ssl102' ]]; then
-                export SSL_VER=".${SSL_VER_TMP}.deb"
-            else
-                export SSL_VER=".${SSL_VER_TMP}"
-            fi
+            export SSL_VER=".${SSL_VER_TMP}"
         fi
     fi
-    #
+    
+    build_mecab_lib
+    build_mecab_dict
+    MECAB_INSTALL_DIR="${WORKDIR}/mecab-install"
     rm -fr TARGET && mkdir TARGET
     rm -rf jemalloc 
     git clone https://github.com/jemalloc/jemalloc
@@ -677,11 +728,15 @@ build_tarball(){
     cd ${TARFILE%.tar.gz}
     if [ "${YASSL}" = 1 ]; then
         DIRNAME="tarball_yassl"
-        bash -xe ./build-ps/build-binary.sh --with-jemalloc=../jemalloc/ --with-yassl ../TARGET
+        CMAKE_OPTS="-DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh --with-jemalloc=../jemalloc/ --with-yassl --with-mecab="${MECAB_INSTALL_DIR}/usr" ../TARGET
     else
-        bash -xe ./build-ps/build-binary.sh --with-jemalloc=../jemalloc/ ../TARGET
+        CMAKE_OPTS="-DWITH_ROCKSDB=1" bash -xe ./build-ps/build-binary.sh --with-mecab="${MECAB_INSTALL_DIR}/usr" --with-jemalloc=../jemalloc/ ../TARGET
+
         DIRNAME="tarball"
     fi
+
+
+    
     mkdir -p ${WORKDIR}/${DIRNAME}
     mkdir -p ${CURDIR}/${DIRNAME}
     cp ../TARGET/*.tar.gz ${WORKDIR}/${DIRNAME}
@@ -691,7 +746,7 @@ build_tarball(){
 #main
 
 CURDIR=$(pwd)
-VERSION_FILE=$CURDIR/percona-server-5.6.properties
+VERSION_FILE=$CURDIR/percona-server-5.7.properties
 args=
 WORKDIR=
 SRPM=0
@@ -709,17 +764,21 @@ INSTALL=0
 RPM_RELEASE=1
 DEB_RELEASE=1
 REVISION=0
-BRANCH="5.6"
+BRANCH="5.7"
+RPM_RELEASE=1
+DEB_RELEASE=1
 YASSL=0
+MECAB_INSTALL_DIR="${WORKDIR}/mecab-install"
 REPO="git://github.com/percona/percona-server.git"
-PRODUCT=Percona-Server-5.6
+PRODUCT=Percona-Server-5.7
 MYSQL_VERSION_MAJOR=5
-MYSQL_VERSION_MINOR=6
-MYSQL_VERSION_PATCH=40
-MYSQL_VERSION_EXTRA=-84.0
-PRODUCT_FULL=Percona-Server-5.6.40-84.0
-PERCONAFT_BRANCH=Percona-Server-5.6.40-84.0
-TOKUBACKUP_BRANCH=Percona-Server-5.6.40-84.0
+MYSQL_VERSION_MINOR=7
+MYSQL_VERSION_PATCH=22
+MYSQL_VERSION_EXTRA=-22
+PRODUCT_FULL=Percona-Server-5.7.22-22
+BOOST_PACKAGE_NAME=boost_1_59_0
+PERCONAFT_BRANCH=Percona-Server-5.7.22-22
+TOKUBACKUP_BRANCH=Percona-Server-5.7.22-22
 parse_arguments PICK-ARGS-FROM-ARGV "$@"
 if [ ${YASSL} = 1 ]; then
   TARBALL=1
@@ -733,5 +792,4 @@ build_tarball
 build_srpm
 build_source_deb
 build_rpm
-build_deb
-
+build_deb
\ No newline at end of file
