@@ -180,6 +180,8 @@ enum enum_sql_command {
   SQLCOM_LOCK_TABLES_FOR_BACKUP,
   SQLCOM_LOCK_BINLOG_FOR_BACKUP,
   SQLCOM_UNLOCK_BINLOG,
+  SQLCOM_CREATE_COMPRESSION_DICTIONARY,
+  SQLCOM_DROP_COMPRESSION_DICTIONARY,
   /* This should be the last !!! */
   SQLCOM_END
 };
