@@ -1,4 +1,6 @@
 /* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2018, Percona and/or its affiliates. All rights reserved.
+   Copyright (c) 2010, 2017, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,10 +20,10 @@
 */
 
 #include "mysys_priv.h"
+#include "my_sys.h"
 #include <m_string.h>
 #include <stdarg.h>
 #include <m_ctype.h>
-#include "mysql/psi/mysql_file.h"
 
 /*
   Copy contents of an IO_CACHE to a file.
@@ -198,10 +200,16 @@ void my_b_seek(IO_CACHE *info,my_off_t pos)
 
 size_t my_b_fill(IO_CACHE *info)
 {
-  my_off_t pos_in_file=(info->pos_in_file+
-			(size_t) (info->read_end - info->buffer));
+  my_off_t pos_in_file;
   size_t diff_length, length, max_length;
 
+  if (info->myflags & MY_ENCRYPT)
+  {
+    DBUG_ASSERT(info->read_pos == info->read_end);
+    return _my_b_read(info, 0, 0) ? 0 : info->read_end - info->read_pos;
+  }
+  pos_in_file= info->pos_in_file + (size_t) (info->read_end - info->buffer);
+
   if (info->seek_not_done)
   {					/* File touched, do seek */
     if (mysql_file_seek(info->file,pos_in_file,MY_SEEK_SET,MYF(0)) ==
@@ -236,6 +244,20 @@ size_t my_b_fill(IO_CACHE *info)
   return length;
 }
 
+int my_b_pread(IO_CACHE *info, uchar *Buffer, size_t Count, my_off_t pos)
+{
+  if (info->myflags & MY_ENCRYPT)
+  {
+    my_b_seek(info, pos);
+    return my_b_read(info, Buffer, Count);
+  }
+
+  /* backward compatibility behavior. XXX remove it? */
+  if (mysql_file_pread(info->file, Buffer, Count, pos,
+                       info->myflags | MY_NABP))
+    return info->error= -1;
+  return 0;
+}
 
 /*
   Read a string ended by '\n' into a buffer of 'max_length' size.
@@ -292,10 +314,15 @@ my_off_t my_b_filelength(IO_CACHE *info)
 }
 
 
-/*
-  Simple printf version.  Supports '%s', '%d', '%u', "%ld", "%lu" and "%llu"
-  Used for logging in MySQL
-  returns number of written character, or (size_t) -1 on error
+/**
+  Simple printf version. Used for logging in MySQL.
+  Supports '%c', '%s', '%b', '%d', '%u', '%ld', '%lu' and '%llu'.
+  Returns number of written characters, or (size_t) -1 on error.
+
+  @param info           The IO_CACHE to write to
+  @param fmt            format string
+  @param ...            variable list of arguments
+  @return               number of bytes written, -1 if an error occurred
 */
 
 size_t my_b_printf(IO_CACHE *info, const char* fmt, ...)
@@ -309,6 +336,15 @@ size_t my_b_printf(IO_CACHE *info, const char* fmt, ...)
 }
 
 
+/**
+  Implementation of my_b_printf.
+
+  @param info           The IO_CACHE to write to
+  @param fmt            format string
+  @param args           variable list of arguments
+  @return               number of bytes written, -1 if an error occurred
+*/
+
 size_t my_b_vprintf(IO_CACHE *info, const char* fmt, va_list args)
 {
   size_t out_length= 0;
@@ -372,7 +408,7 @@ process_flags:
 
     if (*fmt == '*')
     {
-      precision= (int) va_arg(args, int);
+      minimum_width= (int) va_arg(args, int);
       fmt++;
     }
     else
@@ -413,6 +449,7 @@ process_flags:
     {
       char par[2];
       par[0] = va_arg(args, int);
+      out_length++;
       if (my_b_write(info, (uchar*) par, 1))
         goto err;
     }
@@ -445,12 +482,10 @@ process_flags:
           memset(buffz, '0', minimum_width - length2);
         else
           memset(buffz, ' ', minimum_width - length2);
-        if (my_b_write(info, buffz, minimum_width - length2))
+        if (my_b_write(info, (uchar*)buffz, minimum_width - length2))
         {
-          my_afree(buffz);
           goto err;
         }
-        my_afree(buffz);
       }
 
       out_length+= length2;
