@@ -105,7 +105,8 @@ static const char *handler_error_messages[]=
   "Incorrect file name",
   "Operation is not allowed",
   "Compute generate value failed",
-  "Destination schema does not exist"
+  "Destination schema does not exist",
+  "Partitioning can't be initialized"
 };
 
 extern void my_handler_error_register(void);
