@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2013, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -88,13 +88,12 @@ bool One_thread_connection_handler::add_connection(Channel_info* channel_info)
     delete channel_info;
     while (thd_connection_alive(thd))
     {
-      mysql_audit_release(thd);
       if (do_command(thd))
         break;
     }
     end_connection(thd);
   }
-  close_connection(thd);
+  close_connection(thd, 0, false, false);
 
   if (unlikely(opt_userstat))
   {
