@@ -2,7 +2,7 @@
 #define PARTITION_HANDLER_INCLUDED
 
 /*
-   Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -561,7 +561,6 @@ public:
   int ph_rnd_next(uchar *buf);
   void ph_position(const uchar *record);
   int ph_rnd_pos(uchar *buf, uchar *pos);
-  int ph_rnd_pos_by_record(uchar *record);
 
   /** @} */
 
@@ -821,8 +820,6 @@ private:
 
   /** handler to use (ha_partition, ha_innopart etc.) */
   handler *m_handler;
-  /** Convenience pointer to table from m_handler (i.e. m_handler->table). */
-  TABLE *m_table;
 
   /*
     Access methods to protected areas in handler to avoid adding
@@ -906,13 +903,6 @@ private:
   /* If ph_rnd_pos is used then this needs to be implemented! */
   virtual int rnd_pos_in_part(uint part_id, uchar *buf, uchar *pos)
   { DBUG_ASSERT(0); return HA_ERR_WRONG_COMMAND; }
-  virtual int rnd_pos_by_record_in_last_part(uchar *row)
-  {
-    /*
-      Not much overhead to use default function. This avoids out-of-sync code.
-    */
-    return m_handler->rnd_pos_by_record(row);
-  }
   virtual int index_init_in_part(uint part, uint keynr, bool sorted)
   { DBUG_ASSERT(0); return HA_ERR_WRONG_COMMAND; }
   virtual int index_end_in_part(uint part)
@@ -1207,6 +1197,9 @@ private:
     but easier to expose them to derived classes to use.
   */
 protected:
+
+  /** Convenience pointer to table from m_handler (i.e. m_handler->table). */
+  TABLE *m_table;
   /** All internal partitioning data! @{ */
   /** Tables partitioning info (same as table->part_info) */
   partition_info *m_part_info;
