@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -112,6 +112,7 @@ int main(int argc, char *argv[])
   keyinfo[2].seg[0].length=8;
   keyinfo[2].seg[0].null_bit=0;
   keyinfo[2].seg[0].charset=cs;
+  keyinfo[2].seg[0].flag=0;
   keyinfo[3].seg=keyseg+4;
   keyinfo[3].keysegs=1;
   keyinfo[3].flag=HA_NOSAME;
@@ -582,6 +583,7 @@ end:
   heap_clear(file2);
   if (heap_close(file) || (file2 && heap_close(file2)))
     goto err;
+  hp_free(tmp_share);
   heap_delete_table(filename2);
   hp_panic(HA_PANIC_CLOSE);
   my_end(MY_GIVE_INFO);
@@ -641,7 +643,7 @@ static int rnd(int max_value)
 } /* rnd */
 
 
-static void endprog(int sig_number __attribute__((unused)))
+static void endprog(int sig_number MY_ATTRIBUTE((unused)))
 {
   {
     hp_panic(HA_PANIC_CLOSE);
