@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2014, 2016, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -62,8 +62,8 @@ PageBulk::init()
 		the allocation order, and we will always generate redo log
 		for page allocation, even when creating a new tablespace. */
 		mtr_start(&alloc_mtr);
-		if (dict_index_get_space(m_index)
-		    == srv_tmp_space.space_id()) {
+		if (dict_table_is_temporary(m_index->table)) {
+
 			/* We are bulk loading a temporary table index. No need
 			to redo-log it. */
 			alloc_mtr.set_log_mode(MTR_LOG_NO_REDO);
@@ -598,7 +598,7 @@ PageBulk::release()
 }
 
 /** Start mtr and latch the block */
-void
+dberr_t
 PageBulk::latch()
 {
 	ibool	ret;
@@ -618,13 +618,20 @@ PageBulk::latch()
 
 		m_block = buf_page_get_gen(page_id, page_size, RW_X_LATCH,
 					   m_block, BUF_GET_IF_IN_POOL,
-					   __FILE__, __LINE__, m_mtr);
+					   __FILE__, __LINE__, m_mtr, false, &m_err);
+
+		if (m_err != DB_SUCCESS) {
+			return (m_err);
+		}
+
 		ut_ad(m_block != NULL);
 	}
 
 	buf_block_buf_fix_dec(m_block);
 
 	ut_ad(m_cur_rec > m_page && m_cur_rec < m_heap_top);
+
+	return (m_err);
 }
 
 /** Split a page
@@ -770,6 +777,7 @@ BtrBulk::insert(
 	ulint		level)
 {
 	bool		is_left_most = false;
+	dberr_t		err = DB_SUCCESS;
 
 	ut_ad(m_heap != NULL);
 
@@ -778,7 +786,7 @@ BtrBulk::insert(
 		PageBulk*	new_page_bulk
 			= UT_NEW_NOKEY(PageBulk(m_index, m_trx_id, FIL_NULL,
 						level, m_flush_observer));
-		dberr_t	err = new_page_bulk->init();
+		err = new_page_bulk->init();
 		if (err != DB_SUCCESS) {
 			return(err);
 		}
@@ -805,29 +813,37 @@ BtrBulk::insert(
 	ulint		n_ext = 0;
 	ulint		rec_size = rec_get_converted_size(m_index, tuple, n_ext);
 	big_rec_t*	big_rec = NULL;
+	rec_t*		rec = NULL;
+	ulint*		offsets = NULL;
 
 	if (page_bulk->needExt(tuple, rec_size)) {
 		/* The record is so big that we have to store some fields
 		externally on separate database pages */
 		big_rec = dtuple_convert_big_rec(m_index, 0, tuple, &n_ext);
 
-		if (UNIV_UNLIKELY(big_rec == NULL)) {
+		if (big_rec == NULL) {
 			return(DB_TOO_BIG_RECORD);
 		}
 
 		rec_size = rec_get_converted_size(m_index, tuple, n_ext);
 	}
 
+	if (page_bulk->getPageZip() != NULL
+	    && page_zip_is_too_big(m_index, tuple)) {
+		err = DB_TOO_BIG_RECORD;
+		goto func_exit;
+	}
+
 	if (!page_bulk->isSpaceAvailable(rec_size)) {
 		/* Create a sibling page_bulk. */
 		PageBulk*	sibling_page_bulk;
 		sibling_page_bulk = UT_NEW_NOKEY(PageBulk(m_index, m_trx_id,
 							  FIL_NULL, level,
 							  m_flush_observer));
-		dberr_t	err = sibling_page_bulk->init();
+		err = sibling_page_bulk->init();
 		if (err != DB_SUCCESS) {
 			UT_DELETE(sibling_page_bulk);
-			return(err);
+			goto func_exit;
 		}
 
 		/* Commit page bulk. */
@@ -835,7 +851,7 @@ BtrBulk::insert(
 		if (err != DB_SUCCESS) {
 			pageAbort(sibling_page_bulk);
 			UT_DELETE(sibling_page_bulk);
-			return(err);
+			goto func_exit;
 		}
 
 		/* Set new page bulk to page_bulks. */
@@ -849,7 +865,8 @@ BtrBulk::insert(
 		if (page_is_leaf(sibling_page_bulk->getPage())) {
 			/* Check whether trx is interrupted */
 			if (m_flush_observer->check_interrupted()) {
-				return(DB_INTERRUPTED);
+				err = DB_INTERRUPTED;
+				goto func_exit;
 			}
 
 			/* Wake up page cleaner to flush dirty pages. */
@@ -861,8 +878,6 @@ BtrBulk::insert(
 
 	}
 
-	rec_t*		rec;
-	ulint*		offsets = NULL;
 	/* Convert tuple to rec. */
         rec = rec_convert_dtuple_to_rec(static_cast<byte*>(mem_heap_alloc(
 		page_bulk->m_heap, rec_size)), m_index, tuple, n_ext);
@@ -872,8 +887,6 @@ BtrBulk::insert(
 	page_bulk->insert(rec, offsets);
 
 	if (big_rec != NULL) {
-		dberr_t		err;
-
 		ut_ad(dict_index_is_clust(m_index));
 		ut_ad(page_bulk->getLevel() == 0);
 		ut_ad(page_bulk == m_page_bulks->at(0));
@@ -892,13 +905,14 @@ BtrBulk::insert(
 			PageBulk*    page_bulk = m_page_bulks->at(level);
 			page_bulk->latch();
 		}
-
-		dtuple_convert_back_big_rec(m_index, tuple, big_rec);
-
-		return(err);
 	}
 
-	return(DB_SUCCESS);
+func_exit:
+	if (big_rec != NULL) {
+		dtuple_convert_back_big_rec(m_index, tuple, big_rec);
+	}
+
+	return(err);
 }
 
 /** Btree bulk load finish. We commit the last page in each level
@@ -951,8 +965,8 @@ BtrBulk::finish(dberr_t	err)
 					       m_flush_observer);
 
 		mtr_start(&mtr);
-		if (dict_index_get_space(m_index)
-		    == srv_tmp_space.space_id()) {
+		if (dict_table_is_temporary(m_index->table)) {
+
 			/* We are bulk loading a temporary table index. No need
 			to redo-log it. */
 			mtr.set_log_mode(MTR_LOG_NO_REDO);
