@@ -38,6 +38,7 @@ Created 2011/12/19
 #include "srv0srv.h"
 #include "page0zip.h"
 #include "trx0sys.h"
+#include "os0file.h"
 
 #ifndef UNIV_HOTBACKUP
 
@@ -346,8 +347,8 @@ buf_parallel_dblwr_make_path(void)
 	if (parallel_dblwr_buf.path)
 		return(DB_SUCCESS);
 
-	char path[FN_REFLEN];
-	const char *dir;
+	char path[FN_REFLEN + 1 /* OS_PATH_SEPARATOR */];
+	const char *dir = NULL;
 
 	ut_ad(srv_parallel_doublewrite_path);
 
@@ -615,8 +616,7 @@ buf_dblwr_init_or_load_pages(
 			}
 
 		} else {
-
-			recv_dblwr.add(page);
+			recv_dblwr.add_to_sys(page);
 		}
 
 		page += univ_page_size.physical();
@@ -719,10 +719,18 @@ buf_dblwr_init_or_load_pages(
 			return(DB_ERROR);
 		}
 
+		byte zero_page[UNIV_PAGE_SIZE_MAX] = {0};
 		for (page = recovery_buf; page < recovery_buf + size;
 		     page += UNIV_PAGE_SIZE) {
 
-			recv_dblwr.add(page);
+			/* Skip all zero pages */
+			const ulint	checksum = mach_read_from_4(
+				page + FIL_PAGE_SPACE_OR_CHKSUM);
+
+			if (checksum != 0
+                            || memcmp(page, zero_page, UNIV_PAGE_SIZE) != 0) {
+				recv_dblwr.add(page);
+			}
 		}
 		buf_parallel_dblwr_close();
 	}
@@ -779,7 +787,7 @@ buf_dblwr_process(void)
 	     i != recv_dblwr.pages.end();
 	     ++i, ++page_no_dblwr) {
 
-		const byte*	page		= *i;
+		byte*		page		= *i;
 		ulint		page_no		= page_get_page_no(page);
 		ulint		space_id	= page_get_space_id(page);
 
@@ -845,7 +853,13 @@ buf_dblwr_process(void)
 					<< ". Trying to recover it from the"
 					<< " doublewrite buffer.";
 
-				if (buf_page_is_corrupted(
+				dberr_t	err = DB_SUCCESS;
+
+				if (space->crypt_data == NULL) // if it was crypt_data encrypted it was already decrypted
+					err = os_dblwr_decrypt_page(
+					space, page);
+
+				if (err != DB_SUCCESS || buf_page_is_corrupted(
 					true, page, page_size,
 					fsp_is_checksum_disabled(space_id))) {
 
@@ -1113,8 +1127,8 @@ buf_dblwr_check_block(
 		/* TODO: validate also non-index pages */
 		return;
 	case FIL_PAGE_TYPE_ALLOCATED:
-		/* empty pages should never be flushed */
-		break;
+		/* empty pages could be flushed by encryption threads */
+		return;
 	}
 
 	buf_dblwr_assert_on_corrupt_block(block);
@@ -1166,6 +1180,59 @@ buf_dblwr_write_block_to_datafile(
 	}
 }
 
+/** Encrypt a page in doublewerite buffer shard. The page is
+encrypted using its tablespace key.
+@param[in]	block		the buffer pool block for the page
+@param[in,out]	dblwr_page	in: unencrypted page
+				out: encrypted page (if tablespace is
+				encrypted */
+static
+void
+buf_dblwr_encrypt_page(
+	const buf_block_t*	block,
+	page_t*			dblwr_page)
+{
+	const ulint	space_id = block->page.id.space();
+	fil_space_t*	space = fil_space_acquire_silent(space_id);
+
+	if (space == NULL) {
+		/* Tablespace dropped */
+		return;
+	}
+
+	byte*		encrypted_buf = static_cast<byte*>(
+		ut_zalloc_nokey(UNIV_PAGE_SIZE));
+	ut_a(encrypted_buf != NULL);
+
+	const page_size_t	page_size(space->flags);
+	const bool 	success = os_dblwr_encrypt_page(
+		space, dblwr_page, encrypted_buf, UNIV_PAGE_SIZE);
+
+	if (success) {
+		memcpy(dblwr_page, encrypted_buf, page_size.physical());
+	}
+
+	ut_free(encrypted_buf);
+
+	fil_space_release(space);
+}
+
+/* Disable encryption of Page 0 of any tablespace or if it is system
+tablespace, do not encrypt pages upto TRX_SYS_PAGE_NO (including).
+TRX_SYS_PAGE should be not encrypted because dblwr buffer is found
+from this page
+@param[in]	block	buffer block
+@return true if encryption should be disabled for the block, else flase */
+static
+bool
+buf_dblwr_disable_encryption(
+	const buf_block_t*	block)
+{
+	return(block->page.id.page_no() == 0
+	       || (block->page.id.space() == TRX_SYS_SPACE
+		   && block->page.id.page_no() <= TRX_SYS_PAGE_NO));
+}
+
 /********************************************************************//**
 Flushes possible buffered writes from the specified partition of the
 doublewrite memory buffer to disk, and also wakes up the aio thread if
@@ -1210,6 +1277,8 @@ buf_dblwr_flush_buffered_writes(
 
 	write_buf = dblwr_shard->write_buf;
 
+	const bool	encrypt_parallel_dblwr = srv_parallel_dblwr_encrypt;
+
 	for (ulint len2 = 0, i = 0;
 	     i < dblwr_shard->first_free;
 	     len2 += UNIV_PAGE_SIZE, i++) {
@@ -1218,6 +1287,8 @@ buf_dblwr_flush_buffered_writes(
 
 		block = (buf_block_t*)dblwr_shard->buf_block_arr[i];
 
+		page_t*	dblwr_page = write_buf + len2;
+
 		if (buf_block_get_state(block) != BUF_BLOCK_FILE_PAGE
 		    || block->page.zip.data) {
 			/* No simple validate for compressed
@@ -1231,7 +1302,14 @@ buf_dblwr_flush_buffered_writes(
 
 		/* Check that the page as written to the doublewrite
 		buffer has sane LSN values. */
-		buf_dblwr_check_page_lsn(write_buf + len2);
+		buf_dblwr_check_page_lsn(dblwr_page);
+
+		// it can be already encrypted by encryption threads
+		FilSpace space (TRX_SYS_SPACE);
+		if (encrypt_parallel_dblwr && space()->crypt_data == NULL
+		    && !buf_dblwr_disable_encryption(block)) {
+			buf_dblwr_encrypt_page(block, dblwr_page);
+		}
 	}
 
 	len = dblwr_shard->first_free * UNIV_PAGE_SIZE;
@@ -1442,6 +1520,12 @@ retry:
 	write it. This is so because we want to pad the remaining
 	bytes in the doublewrite page with zeros. */
 
+	IORequest	write_request(IORequest::WRITE);
+
+	if (buf_dblwr_disable_encryption((buf_block_t*)bpage)) {
+		write_request.disable_encryption();
+	}
+
 	if (bpage->size.is_compressed()) {
 		memcpy(buf_dblwr->write_buf + univ_page_size.physical() * i,
 		       bpage->zip.data, bpage->size.physical());
@@ -1450,7 +1534,7 @@ retry:
 		       + bpage->size.physical(), 0x0,
 		       univ_page_size.physical() - bpage->size.physical());
 
-		fil_io(IORequestWrite, true,
+		fil_io(write_request, true,
 		       page_id_t(TRX_SYS_SPACE, offset), univ_page_size, 0,
 		       univ_page_size.physical(),
 		       (void*) (buf_dblwr->write_buf
