@@ -106,6 +106,13 @@ flag is cleared and the x-lock released by an i/o-handler thread.
 @param[in] mode		BUF_READ_IBUF_PAGES_ONLY, ...,
 @param[in] page_id	page id
 @param[in] unzip	true=request uncompressed page
+@param[in] should_buffer
+			whether to buffer an aio request.
+			AIO read ahead uses this. If you plan to
+			use this parameter, make sure you remember to
+			call os_aio_dispatch_read_array_submit()
+			when you're ready to commit all your requests.
+
 @return 1 if a read request was queued, 0 if the page already resided
 in buf_pool, or if the page is in the doublewrite buffer blocks in
 which case it is never read into the pool, or if the tablespace does
@@ -120,12 +127,11 @@ buf_read_page_low(
 	const page_id_t&	page_id,
 	const page_size_t&	page_size,
 	bool			unzip,
-	trx_t*			trx)
+	trx_t*			trx,
+	bool			should_buffer)
 {
 	buf_page_t*	bpage;
 
-	ut_ad(!trx || trx->take_stats);
-
 	*err = DB_SUCCESS;
 
 	if (page_id.space() == TRX_SYS_SPACE
@@ -182,16 +188,30 @@ buf_read_page_low(
 		dst = ((buf_block_t*) bpage)->frame;
 	}
 
+	/* This debug code is only for 5.7. In trunk, with newDD,
+	the space->name is no longer same as table name. */
+	DBUG_EXECUTE_IF("innodb_invalid_read_after_truncate",
+		fil_space_t*	space = fil_space_get(page_id.space());
+
+		if (space != NULL && strcmp(space->name, "test/t1") == 0
+		    && page_id.page_no() == space->size - 1) {
+			type = IORequest::READ;
+			sync = true;
+		}
+	);
+
 	IORequest	request(type | IORequest::READ);
 
 	*err = _fil_io(
 		request, sync, page_id, page_size, 0, page_size.physical(),
-		dst, bpage, trx);
+		dst, bpage, trx, should_buffer);
 
 	if (sync) {
 		thd_wait_end(NULL);
 	}
 
+	bpage->is_corrupt = bpage->encrypted = false;
+
 	if (*err != DB_SUCCESS) {
 		if (*err == DB_TABLESPACE_TRUNCATED) {
 			/* Remove the page which is outside the
@@ -209,9 +229,12 @@ buf_read_page_low(
 			   || *err == DB_TABLESPACE_DELETED) {
 			buf_read_page_handle_error(bpage);
 			return(0);
-		}
+		} else if (*err == DB_IO_DECRYPT_FAIL)
+                {
+                        bpage->encrypted= true;
+                }
 
-		SRV_CORRUPT_TABLE_CHECK(*err == DB_SUCCESS,
+		SRV_CORRUPT_TABLE_CHECK(*err == DB_SUCCESS || bpage->encrypted,
 					bpage->is_corrupt = true;);
 
 	}
@@ -219,7 +242,10 @@ buf_read_page_low(
 	if (sync) {
 		/* The i/o is already completed when we arrive from
 		fil_read */
-		if (!buf_page_io_complete(bpage)) {
+
+		*err = buf_page_io_complete(bpage);
+
+		if (*err != DB_SUCCESS) {
 			return(0);
 		}
 	}
@@ -260,8 +286,6 @@ buf_read_ahead_random(
 	const ulint	buf_read_ahead_random_area
 				= BUF_READ_AHEAD_AREA(buf_pool);
 
-	ut_ad(!trx || trx->take_stats);
-
 	if (!srv_random_read_ahead) {
 		/* Disabled by user */
 		return(0);
@@ -326,6 +350,17 @@ buf_read_ahead_random(
 	that is, reside near the start of the LRU list. */
 
 	for (i = low; i < high; i++) {
+		/* This debug code is only for 5.7. In trunk, with newDD,
+		the space->name is no longer same as table name. */
+		DBUG_EXECUTE_IF("innodb_invalid_read_after_truncate",
+			fil_space_t*	space = fil_space_get(page_id.space());
+
+			if (space != NULL
+			    && strcmp(space->name, "test/t1") == 0) {
+				high = space->size;
+				goto read_ahead;
+			}
+		);
 
 		rw_lock_t* hash_lock;
 
@@ -379,14 +414,22 @@ read_ahead:
 				&err, false,
 				IORequest::DO_NOT_WAKE,
 				ibuf_mode,
-				cur_page_id, page_size, false, trx);
+				cur_page_id, page_size, false, trx, false);
 
-			if (err == DB_TABLESPACE_DELETED) {
-				ib::warn() << "Random readahead trying to"
+			switch (err) {
+			case DB_SUCCESS:
+			case DB_TABLESPACE_TRUNCATED:
+			case DB_ERROR:
+			case DB_IO_DECRYPT_FAIL:
+				break;
+			case DB_TABLESPACE_DELETED:
+				ib::info() << "Random readahead trying to"
 					" access page " << cur_page_id
 					<< " in nonexisting or"
 					" being-dropped tablespace";
 				break;
+			default:
+				ut_error;
 			}
 		}
 	}
@@ -419,17 +462,19 @@ an exclusive lock on the buffer frame. The flag is cleared and the x-lock
 released by the i/o-handler thread.
 @param[in]	page_id		page id
 @param[in]	page_size	page size
-@return TRUE if page has been read in, FALSE in case of failure */
-ibool
+@retval DB_SUCCESS if the page was read and is not corrupted,
+@retval DB_PAGE_CORRUPTED if page based on checksum check is corrupted,
+@retval DB_DECRYPTION_FAILED if page post encryption checksum matches but
+after decryption normal page checksum does not match.
+@retval DB_TABLESPACE_DELETED if tablespace .ibd file is missing */
+dberr_t
 buf_read_page(
 	const page_id_t&	page_id,
 	const page_size_t&	page_size,
 	trx_t*			trx)
 {
 	ulint		count;
-	dberr_t		err;
-
-	ut_ad(!trx || trx->take_stats);
+	dberr_t		err = DB_SUCCESS;
 
 	/* We do synchronous IO because our AIO completion code
 	is sub-optimal. See buf_page_io_complete(), we have to
@@ -439,7 +484,7 @@ buf_read_page(
 
 	count = buf_read_page_low(
 		&err, true,
-		0, BUF_READ_ANY_PAGE, page_id, page_size, false, trx);
+		0, BUF_READ_ANY_PAGE, page_id, page_size, false, trx, false);
 
 	srv_stats.buf_pool_reads.add(count);
 
@@ -451,7 +496,7 @@ buf_read_page(
 	/* Increment number of I/O operations used for LRU policy. */
 	buf_LRU_stat_inc_io();
 
-	return(count > 0);
+	return(err);
 }
 
 /** High-level function which reads a page asynchronously from a file to the
@@ -475,7 +520,7 @@ buf_read_page_background(
 		&err, sync,
 		IORequest::DO_NOT_WAKE | IORequest::IGNORE_MISSING,
 		BUF_READ_ANY_PAGE,
-		page_id, page_size, false, NULL);
+		page_id, page_size, false, NULL, false);
 
 	srv_stats.buf_pool_reads.add(count);
 
@@ -539,8 +584,6 @@ buf_read_ahead_linear(
 		= BUF_READ_AHEAD_AREA(buf_pool);
 	ulint		threshold;
 
-	ut_ad(!trx || trx->take_stats);
-
 	/* check if readahead is disabled */
 	if (!srv_read_ahead_threshold) {
 		return(0);
@@ -754,17 +797,33 @@ buf_read_ahead_linear(
 			count += buf_read_page_low(
 				&err, false,
 				IORequest::DO_NOT_WAKE,
-				ibuf_mode, cur_page_id, page_size, false, trx);
+				ibuf_mode, cur_page_id, page_size, false,
+				trx, true);
 
-			if (err == DB_TABLESPACE_DELETED) {
-				ib::warn() << "linear readahead trying to"
+			switch (err) {
+			case DB_TABLESPACE_DELETED:
+                        	ib::warn() << "linear readahead trying to"
 					" access page "
 					<< page_id_t(page_id.space(), i)
 					<< " in nonexisting or being-dropped"
 					" tablespace";
+				break;
+			case DB_SUCCESS:
+			case DB_TABLESPACE_TRUNCATED:
+			case DB_ERROR:
+				break;
+			case DB_PAGE_CORRUPTED:
+			case DB_DECRYPTION_FAILED:
+				ib::error() << "linear readahead failed to"
+					" read or decrypt "
+					<< page_id_t(page_id.space(), i);
+				break;
+			default:
+				ut_error;
 			}
 		}
 	}
+	os_aio_dispatch_read_array_submit();
 
 	/* In simulated aio we wake the aio handler threads only after
 	queuing all aio requests, in native aio the following call does
@@ -841,13 +900,26 @@ buf_read_ibuf_merge_pages(
 				  sync && (i + 1 == n_stored),
 				  0,
 				  BUF_READ_ANY_PAGE, page_id, page_size,
-				  true, NULL);
+				  true, NULL, false);
 
-		if (err == DB_TABLESPACE_DELETED) {
+		switch(err) {
+		case DB_SUCCESS:
+		case DB_TABLESPACE_TRUNCATED:
+		case DB_ERROR:
+			break;
+		case DB_TABLESPACE_DELETED:
 			/* We have deleted or are deleting the single-table
 			tablespace: remove the entries for that page */
 			ibuf_merge_or_delete_for_page(NULL, page_id,
 						      &page_size, FALSE);
+			// fallthrough
+		case DB_PAGE_CORRUPTED:
+		case DB_DECRYPTION_FAILED:
+			ib::error() << "Failed to read or decrypt " << page_id
+				<< " for change buffer merge";
+			break;
+		default:
+			ut_error;
 		}
 	}
 
@@ -918,13 +990,21 @@ buf_read_recv_pages(
 				&err, true,
 				0,
 				BUF_READ_ANY_PAGE,
-				cur_page_id, page_size, true, NULL);
+				cur_page_id, page_size, true, NULL, false);
 		} else {
 			buf_read_page_low(
 				&err, false,
 				IORequest::DO_NOT_WAKE,
 				BUF_READ_ANY_PAGE,
-				cur_page_id, page_size, true, NULL);
+				cur_page_id, page_size, true, NULL, false);
+		}
+
+		if (err == DB_DECRYPTION_FAILED) {
+			ib::error() << "Recovery failed to decrypt page "
+				<< cur_page_id;
+		} else if (err == DB_PAGE_CORRUPTED) {
+			ib::error() << "Recovery failed due to corrupted page "
+				<< cur_page_id;
 		}
 	}
 
