@@ -572,9 +572,22 @@ innobase_get_trx(void);
 /** Get the transaction of the current connection handle if slow query log
 InnoDB extended statistics should be collected.
 @return transaction object if statistics should be collected, or NULL. */
+MY_NODISCARD
 trx_t*
 innobase_get_trx_for_slow_log(void);
 
+extern bool innodb_inited;
+
+enum srv_encrypt_tables_values {
+	SRV_ENCRYPT_TABLES_OFF = 0,
+	SRV_ENCRYPT_TABLES_ON = 1,
+	SRV_ENCRYPT_TABLES_FORCE = 2,
+	SRV_ENCRYPT_TABLES_KEYRING_ON = 3,
+	SRV_ENCRYPT_TABLES_KEYRING_FORCE = 4,
+	SRV_ENCRYPT_TABLES_ONLINE_TO_KEYRING = 5,
+	SRV_ENCRYPT_TABLES_ONLINE_TO_KEYRING_FORCE = 6,
+	SRV_ENCRYPT_TABLES_ONLINE_FROM_KEYRING_TO_UNENCRYPTED = 7
+};
 #endif /* !UNIV_HOTBACKUP && !UNIV_INNOCHECKSUM */
 
 #endif /* HA_INNODB_PROTOTYPES_H */
