@@ -212,7 +212,7 @@ srv_conc_enter_innodb_with_atomics(
 		}
 
 		os_thread_sleep(sleep_in_us);
-		trx->innodb_que_wait_timer += sleep_in_us;
+		trx->stats.bump_innodb_enter_wait(*trx, sleep_in_us);
 
 		trx->op_info = "";
 
