@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2014, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 2014, 2016, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2016, Percona Inc. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -50,7 +50,6 @@ PSI_memory_key	mem_key_log_online_sys;
 PSI_memory_key	mem_key_log_online_read_buf;
 PSI_memory_key	mem_key_log_online_iterator_files;
 PSI_memory_key	mem_key_log_online_iterator_page;
-PSI_memory_key	mem_key_trx_distinct_page_access_hash;
 PSI_memory_key	mem_key_parallel_doublewrite;
 
 #ifdef UNIV_PFS_MEMORY
@@ -83,8 +82,6 @@ static PSI_memory_info	pfs_info[] = {
 	{&mem_key_log_online_read_buf, "log_online_read_buf", 0},
 	{&mem_key_log_online_iterator_files, "log_online_iterator_files", 0},
 	{&mem_key_log_online_iterator_page, "log_online_iterator_page", 0},
-	{&mem_key_trx_distinct_page_access_hash,
-	 "trx_distinct_page_access_hash", 0},
 	{&mem_key_partitioning, "partitioning", 0},
 	{&mem_key_parallel_doublewrite, "parallel_doublewrite", 0},
 };
@@ -196,7 +193,7 @@ ut_new_boot()
 	for (size_t i = 0; i < n_auto; i++) {
 
 		const std::pair<mem_keys_auto_t::iterator, bool>	ret
-			__attribute__((unused))
+			MY_ATTRIBUTE((unused))
 			= mem_keys_auto.insert(
 			mem_keys_auto_t::value_type(auto_event_names[i],
 						    &auto_event_keys[i]));
