@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1994, 2017, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -50,6 +50,10 @@ Created 5/11/1994 Heikki Tuuri
 #include "log.h"
 
 #ifdef _WIN32
+#include <mysql/innodb_priv.h> /* For sql_print_error */
+typedef VOID(WINAPI *time_fn)(LPFILETIME);
+static time_fn ut_get_system_time_as_file_time = GetSystemTimeAsFileTime;
+
 /*****************************************************************//**
 NOTE: The Windows epoch starts from 1601/01/01 whereas the Unix
 epoch starts from 1970/1/1. For selection of constant see:
@@ -57,6 +61,28 @@ http://support.microsoft.com/kb/167296/ */
 #define WIN_TO_UNIX_DELTA_USEC	11644473600000000LL
 
 
+/**
+Initialise highest available time resolution API on Windows
+@return 0 if all OK else -1 */
+int
+ut_win_init_time()
+{
+	HMODULE h = LoadLibrary("kernel32.dll");
+	if (h != NULL)
+	{
+		time_fn pfn = (time_fn)GetProcAddress(h, "GetSystemTimePreciseAsFileTime");
+		if (pfn != NULL)
+		{
+			ut_get_system_time_as_file_time = pfn;
+		}
+		return false;
+	}
+	DWORD error = GetLastError();
+  sql_print_error(
+		"LoadLibrary(\"kernel32.dll\") failed: GetLastError returns %lu", error);
+	return(-1);
+}
+
 /*****************************************************************//**
 This is the Windows version of gettimeofday(2).
 @return 0 if all OK else -1 */
@@ -75,7 +101,7 @@ ut_gettimeofday(
 		return(-1);
 	}
 
-	GetSystemTimeAsFileTime(&ft);
+	ut_get_system_time_as_file_time(&ft);
 
 	tm = (int64_t) ft.dwHighDateTime << 32;
 	tm |= ft.dwLowDateTime;
@@ -225,14 +251,14 @@ ut_print_timestamp(
 
 	GetLocalTime(&cal_tm);
 
-	fprintf(file, "%d-%02d-%02d %02d:%02d:%02d %#lx",
+	fprintf(file, "%d-%02d-%02d %02d:%02d:%02d %#llx",
 		(int) cal_tm.wYear,
 		(int) cal_tm.wMonth,
 		(int) cal_tm.wDay,
 		(int) cal_tm.wHour,
 		(int) cal_tm.wMinute,
 		(int) cal_tm.wSecond,
-		thread_id);
+		static_cast<ulonglong>(thread_id));
 #else
 	struct tm* cal_tm_ptr;
 	time_t	   tm;
@@ -403,10 +429,10 @@ ut_print_buf(
 
 	UNIV_MEM_ASSERT_RW(buf, len);
 
-	fprintf(file, " len %lu; hex ", len);
+	fprintf(file, " len " ULINTPF "; hex ", len);
 
 	for (data = (const byte*) buf, i = 0; i < len; i++) {
-		fprintf(file, "%02lx", (ulong)*data++);
+		fprintf(file, "%02lx", static_cast<ulong>(*data++));
 	}
 
 	fputs("; asc ", file);
@@ -824,16 +850,16 @@ ut_strerr(
 		       "transaction");
 	case DB_WRONG_FILE_NAME:
 		return("Invalid Filename");
-	case DB_NO_FK_ON_V_BASE_COL:
+	case DB_NO_FK_ON_S_BASE_COL:
 		return("Cannot add foreign key on the base column "
-		       "of indexed virtual column");
-	case DB_NO_VIRTUAL_INDEX_ON_FK:
-		return("Cannot create index on virtual column whose base "
-		       "column has foreign constraint");
-
+		       "of stored column");
 	case DB_COMPUTE_VALUE_FAILED:
 		return("Compute generated column failed");
-
+        case DB_DECRYPTION_FAILED:
+		return("Table is encrypted but decrypt failed.");
+	case DB_PAGE_CORRUPTED:
+		return("Page read from tablespace is corrupted.");
+        
 	/* do not add default: in order to produce a warning if new code
 	is added to the enum but not added here */
 	}
@@ -936,6 +962,15 @@ fatal_or_error::~fatal_or_error()
 	ut_a(!m_fatal);
 }
 
+warn_or_info::~warn_or_info()
+{
+	if (m_warn) {
+		sql_print_warning("InnoDB: %s", m_oss.str().c_str());
+	} else {
+		sql_print_information("InnoDB: %s", m_oss.str().c_str());
+	}
+}
+
 } // namespace ib
 
 #endif /* !UNIV_INNOCHECKSUM */
