@@ -3,20 +3,13 @@
     All rights reserved. Use is subject to license terms.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -28,8 +21,6 @@
 #include "SCI_Transporter.hpp" 
 #include <NdbOut.hpp> 
 #include <NdbSleep.h> 
-#include <NdbTick.h> 
-#include <NdbTick.h> 
 
 #include "TransporterInternalDefinitions.hpp" 
 #include <TransporterCallback.hpp> 
@@ -322,10 +313,6 @@ sci_error_t SCI_Transporter::initLocalSegment() {
  
  
 bool SCI_Transporter::doSend() { 
-#ifdef DEBUG_TRANSPORTER  
-  NDB_TICKS startSec=0, stopSec=0; 
-  Uint32 startMicro=0, stopMicro=0, totalMicro=0; 
-#endif
   sci_error_t             err; 
   Uint32 retry=0; 
  
